//
//  SignUpViewController.swift
//  NearbyStores
//
//  Created by DT Team on 5/20/18.
//  Copyright © 2018 Amine. All rights reserved.
//

import UIKit
import SVProgressHUD
import SwiftEventBus
import SwiftIcons
import SkyFloatingLabelTextField
import Kingfisher
import Firebase
import FirebaseAuth
import RealmSwift


class EditProfileViewController: UIViewController, UserLoaderDelegate,  UITextFieldDelegate, UIImagePickerControllerDelegate, UINavigationControllerDelegate {
    
    @IBOutlet weak var saveBtn: CustomButton!
    
    @IBOutlet weak var navigationBar: UINavigationBar!
    @IBOutlet weak var navigationBarItem: UINavigationItem!
    
    
    @IBOutlet weak var constraintScrollBottom: NSLayoutConstraint!
    @IBOutlet weak var scrollview: UIScrollView!
    @IBOutlet weak var stackView: UIStackView!
    @IBOutlet weak var full_name_field: SkyFloatingLabelTextField!
    @IBOutlet weak var email_field: SkyFloatingLabelTextField!
    @IBOutlet weak var login_field: SkyFloatingLabelTextField!
    @IBOutlet weak var password_field: SkyFloatingLabelTextField!
    @IBOutlet weak var pick_photo_btn: UIButton!
    @IBOutlet weak var photo_profile: UIImageView!
    
    @IBAction func onPickPhotoAction(_ sender: Any) {
        
        imagePicker.allowsEditing = false
        imagePicker.sourceType = .photoLibrary
        
        self.present(imagePicker, animated: true)
        
    }
    
    
    @IBAction func saveChangesAction(_ sender: Any) {
        
        doSaveChanges()
        
    }
    
    let topBarTitle: EdgeLabel = {
        
        let titleLabel = EdgeLabel()
        
        titleLabel.text = ""
        titleLabel.textColor = UIColor.white
        titleLabel.font = UIFont.systemFont(ofSize: 20)
        
        return titleLabel
        
    }()
    
    func setupNavBarTitles() {
        
        
        let rect = CGRect(x: 0, y: 0, width: view.frame.width - 32, height: view.frame.height)
        topBarTitle.frame = rect
        topBarTitle.textColor = AppDesignUtils.defaultModeColor(dark: .white, light: Colors.primaryColor)
        topBarTitle.font = UIFont(name: AppConfig.Design.Fonts.regular, size: 17)
        topBarTitle.leftTextInset = 5
        
        
        
        topBarTitle.text = "Edit Profile".localized
        
        navigationBarItem.titleView = topBarTitle
        
    }
    
    
    
    func setupNavBarButtons() {
        
        let color = AppDesignUtils.defaultModeColor(dark: .white, light: Colors.primaryColor)
        
        //arrow back icon
        var arrowImage: UIImage? = nil
        if Utils.isRTL(){
            arrowImage = UIImage.init(icon: .ionicons(.iosArrowForward), size: CGSize(width: 30, height: 30), textColor: color)
        }else{
            arrowImage = UIImage.init(icon: .ionicons(.iosArrowBack), size: CGSize(width: 30, height: 30), textColor: color)
        }
        
        
        
        let customBarButtonItem = UIBarButtonItem(image: arrowImage!, style: .plain, target: self, action: #selector(onBackHandler))
        customBarButtonItem.setIcon(icon: .ionicons(.iosArrowBack), iconSize: 25, color: color)
        
        
        navigationBarItem.leftBarButtonItems = []
        navigationBarItem.rightBarButtonItems = []
        navigationBarItem.leftBarButtonItems?.append(customBarButtonItem)
        
    }
    
    @objc func onBackHandler()  {
        self.dismiss(animated: true)
    }
    
    
    var imageId: String = ""
    
    let imagePicker = UIImagePickerController()
    
    var myUserSession: User? = nil
    
    
    @IBOutlet weak var top_rounded_view: UIView!
    @IBOutlet weak var bottom_rounded_view: UIView!
    
    
    @IBOutlet weak var top_rounded_view0: UIView!
    @IBOutlet weak var bottom_rounded_view0: UIView!
    
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        
        
        self.top_rounded_view.roundCorners(corners: [.topLeft, .topRight], radius: 20)
        self.bottom_rounded_view.roundCorners(corners: [.bottomLeft, .bottomRight], radius: 20)
        
        self.top_rounded_view0.roundCorners(corners: [.topLeft, .topRight], radius: 20)
        self.bottom_rounded_view0.roundCorners(corners: [.bottomLeft, .bottomRight], radius: 20)
        
        
        if let session = Session.getInstance(), let user = session.user{
            myUserSession = user
        }else{
            dismiss(animated: true)
        }
        
        
        imagePicker.delegate = self
        
        if Session.isLogged() {
            
        }
        
        self.navigationBar.isTranslucent = false
        self.navigationBar.setBackgroundImage(UIImage(), for: UIBarMetrics.default)
        self.navigationBar.shadowImage = UIImage()
        self.navigationBar.tintColor = AppDesignUtils.defaultModeColor(dark: .white, light: Colors.primaryColor)
        
        setupNavBarTitles()
        setupNavBarButtons()
        
        setupView()
        
        
        NotificationCenter.default.addObserver(self, selector: #selector(handleKeyboardNotification), name: UIResponder.keyboardWillHideNotification, object: nil)
        
        NotificationCenter.default.addObserver(self, selector: #selector(handleKeyboardNotification), name: UIResponder.keyboardWillShowNotification, object: nil)
        
    }
    
    @objc func handleKeyboardNotification(notification: NSNotification) {
        
        
        if let userInfo = notification.userInfo {
            
            let keyboardFrame = userInfo[UIResponder.keyboardFrameEndUserInfoKey] as! CGRect
            let isKeyboardShowing = notification.name == UIResponder.keyboardWillShowNotification
            
            
            if let textField = self.lastTextField, let _ = textField.superview?.convert(textField.frame, to: nil) {
                
                if isKeyboardShowing{
                    // so increase contentView's height by keyboard height
                    self.constraintScrollBottom.constant = keyboardFrame.height
                    UIView.animate(withDuration: 0, delay: 0, options: UIView.AnimationOptions.curveEaseOut, animations: {
                        self.view.layoutIfNeeded()
                    })
                    
                }else{
                    // so increase contentView's height by keyboard height
                    self.constraintScrollBottom.constant = 0
                    UIView.animate(withDuration: 0, delay: 0, options: UIView.AnimationOptions.curveEaseOut, animations: {
                        self.view.layoutIfNeeded()
                    })
                    
                }
                
                
                
            }
            
            
            
        }
        
        
    }
    
    
    
    @IBAction func haveAccountBtn(_ sender: Any) {
        
        dismiss(animated: true, completion: nil)
    }
    
    
    @IBOutlet weak var stackView_phoneNumber: UIStackView!
    @IBOutlet weak var stackViewUser: UIStackView!
    
    
    @IBOutlet weak var phone_number_field: SkyFloatingLabelTextField!
    @IBOutlet weak var code_verification_field: SkyFloatingLabelTextField!
    
    
    @IBOutlet weak var verify_code_btn: CustomButton!
    
    @IBOutlet weak var code_verification_field_container: UIView!
    @IBOutlet weak var verify_code_btn_container: UIView!
    
 
    
    @IBAction func verify_code_action(_ sender: Any) {
        
        guard let code_v_field = code_verification_field.text else{
            return
        }
        
        if( code_v_field == ""){
            let message: [String:String] = ["alert":"The code verification field is empty!".localized]
            self.showAlertError(title: "Alert!".localized, content: message, msgBnt: "OK".localized)
            return
        }
        
        MyProgress.show(parent: view)
        
        let verificationID =  LocalData.getValue(key: "authVerificationID", defaultValue: "")
        let phoneNumber =  LocalData.getValue(key: "phoneNumber", defaultValue: "")
      
        let credential = PhoneAuthProvider.provider().credential(withVerificationID: verificationID ,
                                                                       verificationCode: code_v_field)
            Auth.auth().signInAndRetrieveData(with: credential) { authData, error in
                
                MyProgress.dismiss()
                
             if let error = error {
                let message: [String:String] = ["AuthError": error.localizedDescription]
                self.showAlertError(title: "Alert!".localized, content: message, msgBnt: "OK".localized)
                return
              }
                
             //update phone number
             Utils.printDebug("\( authData!.user)")
                
            self.update_phone(phone: phoneNumber)
                
         };
        
    }
    
    func update_phone(phone: String) {
        
        guard let user = self.myUserSession else {
            return
        }
        
     
        MyProgress.showProgressWithSuccess(withStatus: "Success!".localized)
        
        
        let api = SimpleLoader()
        
        
        let parameters = [
            "user_id"       : String(user.id),
            "phone"       : String(phone),
        ]
        
        api.run(url: Constances.Api.API_UPDATE_PHONE, parameters: parameters) { (parser) in
            
            MyProgress.dismiss()
            
            if parser?.success == 1{
                
                let realm = try! Realm()
                realm.beginWrite()
                
                if let user = self.myUserSession{
                    user.phone = phone
                }
                
                try! realm.commitWrite()
                
                self.myUserSession?.save()
            
                self.onBackHandler()
               
            }
        }
       

        
    }
    
    func get_code() {
        
        
        guard let phone_field = phone_number_field.text, phone_field != "" else{
            phone_number_field.errorMessage = "Invalid phone".localized
            return
        }
        
        MyProgress.show(parent: self.view)
        
        let phone = phone_field.replacingOccurrences(of: " ", with: "", options: .literal, range: nil)
        let result = phone.isValid(regex: .phone)
        
        if !result {
            phone_number_field.errorMessage = "Invalid phone".localized
            return
        }
        

        PhoneAuthProvider.provider().verifyPhoneNumber(phone) { (verificationID,error) in
            
            MyProgress.dismiss()
            
            if let error = error {
                let message: [String:String] = ["AuthError": error.localizedDescription]
                self.showAlertError(title: "Alert!".localized, content: message, msgBnt: "OK".localized)
                return
            }
            
            // Sign in using the verificationID and the code sent to the user
            // ...
            
            LocalData.setValue(key: "authVerificationID", value: verificationID!)
            LocalData.setValue(key: "phoneNumber", value: phone)
            
            self.code_verification_field_container.isHidden = false
            self.verify_code_btn_container.isHidden = false
            
            self.display_phone_verification()
            
        }
    }
    
    
    func setup_phone_number_component() {
        
      
        code_verification_field.keyboardType = .numberPad
        code_verification_field.textColor = UIColor.black
        code_verification_field.lineColor = UIColor.gray
        code_verification_field.selectedTitleColor = UIColor.black
        code_verification_field.selectedLineColor = Colors.primaryColor
        code_verification_field.selectedLineHeight = 1.5
        code_verification_field.lineHeight = 0.5
        code_verification_field.titleColor = Colors.primaryColor
        code_verification_field.font = UIFont(name: AppConfig.Design.Fonts.regular, size: code_verification_field.font!.pointSize)
        code_verification_field.titleFont = UIFont(name: AppConfig.Design.Fonts.regular, size: code_verification_field.font!.pointSize)!
    
        code_verification_field.delegate = self
        
        
        code_verification_field.inputAccessoryView = setupDoneButtonOnKeyboard()
        
        code_verification_field.placeholder = "Code verification".localized
       
       // get_code_btn.setCustomButtonControlStyle(active: false)
        verify_code_btn.setCustomButtonControlStyle(active: true)
        
        stackView_phoneNumber.isHidden = true
        
        code_verification_field_container.isHidden = true
        verify_code_btn_container.isHidden = true
        
    }
    
    func setupDoneButtonOnKeyboard() -> UIToolbar
    {
        let doneToolbar: UIToolbar = UIToolbar(frame: CGRect(x:0, y:0, width:320, height:50))
        doneToolbar.barStyle = UIBarStyle.default

     let flexSpace = UIBarButtonItem(barButtonSystemItem: UIBarButtonItem.SystemItem.flexibleSpace, target: nil, action: nil)
     let done: UIBarButtonItem = UIBarButtonItem(title: "Done".localized, style: UIBarButtonItem.Style.done, target: self, action: #selector(doneButtonAction))

        let items = NSMutableArray()
        items.add(flexSpace)
        items.add(done)

        doneToolbar.items = items as? [UIBarButtonItem]
        doneToolbar.sizeToFit()
        
        return doneToolbar
       
    }
    
    @objc func doneButtonAction()
    {
        phone_number_field.resignFirstResponder()
        full_name_field.resignFirstResponder()
        email_field.resignFirstResponder()
        password_field.resignFirstResponder()
        login_field.resignFirstResponder()
        code_verification_field.resignFirstResponder()
    }
    
    func setupView ()  {
        
        
        if let value = SyncAppConfig.findById(id: "USER_PHONE_VERIFICATION"), value.getValue(defaultValue: true) == true  {
            setup_phone_number_component()
        }else{
            stackView_phoneNumber.isHidden = true
        }
       
        
        phone_number_field.keyboardType = .phonePad
        phone_number_field.textColor = UIColor.black
        phone_number_field.lineColor = UIColor.gray
        phone_number_field.selectedTitleColor = UIColor.black
        phone_number_field.selectedLineColor = Colors.primaryColor
        phone_number_field.selectedLineHeight = 1.5
        phone_number_field.lineHeight = 0.5
        phone_number_field.titleColor = Colors.primaryColor
        phone_number_field.font = UIFont(name: AppConfig.Design.Fonts.regular, size: phone_number_field.font!.pointSize)
        phone_number_field.titleFont = UIFont(name: AppConfig.Design.Fonts.regular, size: phone_number_field.font!.pointSize)!
        
        phone_number_field.placeholder = "Phone number".localized
        phone_number_field.delegate = self
        
        
        phone_number_field.inputAccessoryView = setupDoneButtonOnKeyboard()
        
        
        full_name_field.textColor = UIColor.black
        full_name_field.lineColor = UIColor.gray
        full_name_field.selectedTitleColor = UIColor.black
        full_name_field.selectedLineColor = Colors.primaryColor
        full_name_field.selectedLineHeight = 1.5
        full_name_field.lineHeight = 0.5
        full_name_field.titleColor = Colors.primaryColor
        
        full_name_field.font = UIFont(name: AppConfig.Design.Fonts.regular, size: login_field.font!.pointSize)
        full_name_field.titleFont = UIFont(name: AppConfig.Design.Fonts.regular, size: password_field.font!.pointSize)!
               
        full_name_field.inputAccessoryView = setupDoneButtonOnKeyboard()
        
        
        email_field.textColor = UIColor.black
        email_field.lineColor = UIColor.gray
        email_field.selectedTitleColor = UIColor.black
        email_field.selectedLineColor = Colors.primaryColor
        email_field.selectedLineHeight = 1.5
        email_field.lineHeight = 0.5
        email_field.titleColor = Colors.primaryColor
        
        email_field.font = UIFont(name: AppConfig.Design.Fonts.regular, size: login_field.font!.pointSize)
        email_field.titleFont = UIFont(name: AppConfig.Design.Fonts.regular, size: password_field.font!.pointSize)!
         
        email_field.inputAccessoryView = setupDoneButtonOnKeyboard()
        
        
        login_field.textColor = UIColor.black
        login_field.lineColor = UIColor.gray
        login_field.selectedTitleColor = UIColor.black
        login_field.selectedLineColor = Colors.primaryColor
        login_field.selectedLineHeight = 1.5
        login_field.lineHeight = 0.5
        login_field.titleColor = Colors.primaryColor
        
        login_field.font = UIFont(name: AppConfig.Design.Fonts.regular, size: login_field.font!.pointSize)
        login_field.titleFont = UIFont(name: AppConfig.Design.Fonts.regular, size: password_field.font!.pointSize)!
         
        login_field.inputAccessoryView = setupDoneButtonOnKeyboard()
        
        
        password_field.textColor = UIColor.black
        password_field.lineColor = UIColor.gray
        password_field.selectedTitleColor = UIColor.black
        password_field.selectedLineColor = Colors.primaryColor
        password_field.selectedLineHeight = 1.5
        password_field.lineHeight = 0.5
        password_field.titleColor = Colors.primaryColor
        
        password_field.font = UIFont(name: AppConfig.Design.Fonts.regular, size: login_field.font!.pointSize)
        password_field.titleFont = UIFont(name: AppConfig.Design.Fonts.regular, size: password_field.font!.pointSize)!
         
        
        password_field.inputAccessoryView = setupDoneButtonOnKeyboard()
       
        
        full_name_field.placeholder = "Enter full name".localized
        email_field.placeholder = "Enter email address".localized
        login_field.placeholder = "Enter login".localized
        password_field.placeholder = "Enter password".localized
        
        saveBtn.setTitle("Save Changes".localized.uppercased(), for: .normal)
        
        
        full_name_field.delegate = self
        email_field.delegate = self
        login_field.delegate = self
        password_field.delegate = self
       
        saveBtn.setCustomButtonControlStyle(active: true)
        saveBtn.setGradientBackground()
      
        /*
         Setup Image Profile
         */
        
        self.photo_profile.roundCorners(radius: self.photo_profile.frame.height/2)
        self.photo_profile.layer.borderColor = Utils.hexStringToUIColor(hex: "#eeeeee").cgColor
        self.pick_photo_btn.setTitle("", for: .normal)
        self.pick_photo_btn.roundCorners(radius: self.photo_profile.frame.height/2)
        let camera = UIImage.init(icon:  .linearIcons(.camera), size: CGSize(width: 36, height: 36), textColor: Colors.white).withRenderingMode(.alwaysOriginal)
        self.pick_photo_btn.setImage(camera, for: .normal)
        self.pick_photo_btn.backgroundColor = UIColor.black.withAlphaComponent(0.3)
        
        
        //put data inside views
        
        if let user = myUserSession {
            
            full_name_field.text = user.name
            login_field.text = user.username
            email_field.text = user.email
            phone_number_field.text = user.phone
            
            //set image
            if let cached_image = EditProfileViewController.imageProfile{
                
                self.photo_profile.image = cached_image
                
            }else if let cached_image = SignUpViewController.imageProfile{
                
                self.photo_profile.image = cached_image
                
            }else{
                
                if let image = user.images {
                    
                    let url = URL(string: image.url100_100)
                    
                    self.photo_profile.kf.indicatorType = .activity
                    self.photo_profile.kf.setImage(with: url,options: [.transition(.fade(0.2))])
                    
                }else{
                    if let img = UIImage(named: "profile_placeholder") {
                        self.photo_profile.image = img
                    }
                }
                
            }
            
            
        }
       
    
        
    }
    
    
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        self.view.endEditing(true)
    }
    
    var lastTextField: UITextField? = nil
    
    func textFieldDidBeginEditing(_ textField: UITextField) {
        
        lastTextField = textField
    }
    
    
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        textField.resignFirstResponder()
        lastTextField = textField
        return true
    }
    
    
    
    var userLoader: UserLoader = UserLoader()
    func doSaveChanges () {
        
        
        guard let user = myUserSession else { return }
        
        
        MyProgress.showProgressWithSuccess(withStatus: "Success!".localized)
        
        self.userLoader.delegate = self
        
        
        guard let loginValue = login_field.text else {
            showErros(messages:["login":"Login field is empty".localized])
            return
        }
        
        guard let passwordValue = password_field.text else {
            showErros(messages:["password":"Password field is empty".localized])
            return
        }
        
        guard let emailValue = email_field.text else {
            showErros(messages:["email":"Email field is empty".localized])
            return
        }
        
        guard let nameValue = full_name_field.text else {
            showErros(messages:["name":"Full Name field is empty".localized])
            return
        }
        
        var lat = 0.0
        var lng = 0.0
        var guest_id = 0
        
        if let guest = Guest.getInstance() {
            
            guest_id = guest.id
            lat = guest.lat
            lng = guest.lng
            
        }
        
        
        self.userLoader.load(url: Constances.Api.API_UPDATE_ACCOUNT,parameters: [
            "username"      : loginValue,
            "oldUsername"   : user.username,
            
            "user_id"       : String(user.id),
            
            "email"         : emailValue,
            "name"          : nameValue,
            "password"      : passwordValue,
            "image"         : imageId,
            "lat"           : String(lat),
            "lng"           : String(lng),
            "guest_id"      : String(guest_id)
            ])
        
    }
    
    
    func success(parser: UserParser,response: String) {
        
        if parser.success == 1 {
            
            
            let users = parser.parse()
            
            if users.count == 1 {
                
                Utils.printDebug("\(users)")
                
                Session.createSession(user: users[0])
                
                SwiftEventBus.post("on_main_refresh", sender: users[0])
                
                //upload image
                if let image = EditProfileViewController.imageProfile{
                    
                    let parameters = [
                        
                        "image":    image.toBase64(),
                        "int_id":   String(users[0].id),
                        "type":     "user",
                        
                        "module":     "user",
                        "module_id":   String(users[0].id),
                    ]
                    
                    let api = SimpleLoader()
                    api.run(url: Constances.Api.API_USER_UPLOAD64, parameters: parameters) { (parser) in
                        if parser?.success == 1{
                            
                        }
                    }
                    
                }
                
                
                MyProgress.dismiss()
                
                if let value = SyncAppConfig.findById(id: "USER_PHONE_VERIFICATION"), value.getValue(defaultValue: true) {
                    
                    guard let phone_field = phone_number_field.text, phone_field != "" else{
                        phone_number_field.errorMessage = "Invalid phone".localized
                        return
                    }
                    
                    if let user = myUserSession, (user.phone != phone_field || user.phone_verified == 0) {
                        get_code()
                    }
                    
                }
        
                
            }else{
                showErros(messages: ["err": "User not found!".localized])
            }
            
            
        }else {
            
            if let errors = parser.errors {
                
                MyProgress.dismiss()
                self.showAlertError(title: "Error",content: errors ,msgBnt: "OK")
            }
            
        }
        
    }
    
    func display_phone_verification()  {
      
        stackView_phoneNumber.isHidden = false
        stackViewUser.isHidden = true
        
    }
    
    func error(error: Error?,response: String) {
        
        MyProgress.dismiss()
        
        let errors: [String: String] = [
            "err": "Technical error"
        ]
        
        self.showAlertError(title: "Error",content: errors,msgBnt: "OK")
        
    }
    
    
    func showErros(messages: [String: String]) {
        
        self.showAlertError(title: "Error",content: messages,msgBnt: "OK")
        
    }
    
    
    func startMainVC() {
        
        let sb = UIStoryboard(name: "Main", bundle: nil)
        if let vc = sb.instantiateInitialViewController() {
            self.present(vc, animated: true)
        }
        
    }
    
    static var imageProfile:UIImage? = nil
    
    func imagePickerController(_ picker: UIImagePickerController, didFinishPickingMediaWithInfo info: [UIImagePickerController.InfoKey : Any]) {
        
        // Local variable inserted by Swift 4.2 migrator.
        let info = convertFromUIImagePickerControllerInfoKeyDictionary(info)

        
        if let pickedImage = info[convertFromUIImagePickerControllerInfoKey(UIImagePickerController.InfoKey.originalImage)] as? UIImage {
            
            if let imageData = pickedImage.jpeg(.lowest) {
                EditProfileViewController.imageProfile = UIImage(data: imageData)
                EditProfileViewController.imageProfile = EditProfileViewController.imageProfile?.fixOrientation()
                self.photo_profile.contentMode = .scaleAspectFill
                self.photo_profile.image = EditProfileViewController.imageProfile
            }
        }
        
        picker.dismiss(animated: true)
        
    }
    
    
    
    
}






// Helper function inserted by Swift 4.2 migrator.
fileprivate func convertFromUIImagePickerControllerInfoKeyDictionary(_ input: [UIImagePickerController.InfoKey: Any]) -> [String: Any] {
	return Dictionary(uniqueKeysWithValues: input.map {key, value in (key.rawValue, value)})
}

// Helper function inserted by Swift 4.2 migrator.
fileprivate func convertFromUIImagePickerControllerInfoKey(_ input: UIImagePickerController.InfoKey) -> String {
	return input.rawValue
}

