//
//  LoginViewController.swift
//  NearbyStores
//
//  Created by DT Team on 5/20/18.
//  Copyright © 2018 Amine. All rights reserved.
//

import UIKit
import SwiftEventBus
import SkyFloatingLabelTextField

protocol LoginControllerDelegate {
    func loginSuccess(controller: LoginViewController, user: User)
    func loginFaild(controller: LoginViewController)
}

class LoginViewController: UIViewController,UserLoaderDelegate, UITextFieldDelegate, SignUpControllerDelegate {
    
      static func newInstance() -> LoginViewController{
          let login_sb = UIStoryboard(name: "Login", bundle: nil)
          let login_vc: LoginViewController = login_sb.instantiateViewController(withIdentifier: "loginVC") as! LoginViewController
          return login_vc
      }
      
      func signupSuccess(controller: SignUpViewController, user: User) {
          
          controller.onBackHandler()
          
          if let del = delegate{
               self.login_field.text = user.username
              del.loginSuccess(controller: self, user: user)
          }else{
            
          }
          
      }
      
      func signupFaild(controller: SignUpViewController) {
          
      }
      
     
    var hasBack = true
    var delegate: LoginControllerDelegate? = nil
    
    
    @IBOutlet weak var navigationBar: UINavigationBar!
    @IBOutlet weak var navigationBarItem: UINavigationItem!
      
   
    @IBOutlet weak var bottomRounded: UIView!
    @IBOutlet weak var logo_header_container: UIView!
    
    @IBOutlet weak var constraintScrollBottomHeight: NSLayoutConstraint!
    @IBOutlet weak var scrollview: UIScrollView!
    @IBOutlet weak var forgotpassword: UIButton!
    @IBOutlet weak var login_field: SkyFloatingLabelTextField!
    @IBOutlet weak var password_field: SkyFloatingLabelTextField!
    @IBOutlet weak var signin: CustomButton!
    @IBOutlet weak var icons_bg: UIImageView!
    
 
    @IBOutlet weak var signup: UIButton!
    @IBAction func signInAction(_ sender: Any) {
    
        dologin()
    
    }
    
    @IBAction func forgotPasswordAction(_ sender: Any) {
        
        
        if let url = URL(string: AppConfig.Api.base_url+"/fpassword"), UIApplication.shared.canOpenURL(url) {
            if #available(iOS 10, *) {
                UIApplication.shared.open(url)
            } else {
                UIApplication.shared.openURL(url)
            }
        }
        
        
        
    }
    @IBAction func signUpAction(_ sender: Any) {
        
        startSignUpView()
    }
    

    
     @objc func onBackHandler()  {
        
        if !hasBack{
            return
        }
              
           if let controller = self.navigationController{
                  //controller.navigationBar.isHidden = true
                  controller.popViewController(animated: true)
                  ////controller.navigationBar.isHidden = false
              }else{
                  self.dismiss(animated: true)
              }
              
       }
    
    func startSignUpView()  {
        
        
        let sb = UIStoryboard(name: "Signup", bundle: nil)
        let vc: SignUpViewController = sb.instantiateViewController(withIdentifier: "signupVC") as! SignUpViewController
        
        vc.delegate = self
        
        self.present(vc, animated: true, completion: nil)
        
    }
    
   

    @IBOutlet weak var top_rounded_view: UIView!
    @IBOutlet weak var bottom_rounded_view: UIView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
    

        self.top_rounded_view.roundCorners(corners: [.topLeft, .topRight], radius: 20)
        self.bottom_rounded_view.roundCorners(corners: [.bottomLeft, .bottomRight], radius: 20)
        
        
        DispatchQueue.main.asyncAfter(deadline: .now()+0.1) {
            self.logo_header_container.setGradientBackground()
        }
       

    
        icons_bg.image = UIImage(named: "icon_bg")?.withRenderingMode(.alwaysTemplate)
        icons_bg.tintColor = Colors.darkColor
        
        if Session.isLogged() {
            DispatchQueue.main.asyncAfter(deadline: .now()+1.0) {
                self.dismiss(animated: true)
            }
        }

        setupViews()
        
        NotificationCenter.default.addObserver(self, selector: #selector(handleKeyboardNotification), name: UIResponder.keyboardWillHideNotification, object: nil)
        
        NotificationCenter.default.addObserver(self, selector: #selector(handleKeyboardNotification), name: UIResponder.keyboardWillShowNotification, object: nil)
        
        SwiftEventBus.onMainThread(self, name: "on_close_it") { result in
            
            if let _ = result?.object{
                
                if let mySession = Session.getInstance(), let user = mySession.user{
                    SwiftEventBus.post("on_main_refresh", sender: user)
                    self.dismiss(animated: true)
                }
            }
            
        }
        
        
        if !hasBack{
            navigationBar.isHidden = true
        }
       
      
    }
    
    
    @objc func handleKeyboardNotification(notification: NSNotification) {
        
        
        if let userInfo = notification.userInfo {
            
            let keyboardFrame = userInfo[UIResponder.keyboardFrameEndUserInfoKey] as! CGRect
            let isKeyboardShowing = notification.name == UIResponder.keyboardWillShowNotification
            
            
            if let textField = self.lastTextField, let _ = textField.superview?.convert(textField.frame, to: nil) {
                
                if isKeyboardShowing{
                    // so increase contentView's height by keyboard height
                    self.constraintScrollBottomHeight.constant = keyboardFrame.height
                    UIView.animate(withDuration: 0, delay: 0, options: UIView.AnimationOptions.curveEaseOut, animations: {
                        self.view.layoutIfNeeded()
                    })
                    
                }else{
                    // so increase contentView's height by keyboard height
                    self.constraintScrollBottomHeight.constant = 0
                    UIView.animate(withDuration: 0, delay: 0, options: UIView.AnimationOptions.curveEaseOut, animations: {
                        self.view.layoutIfNeeded()
                    })
                    
                }
                
            }
        
        }
    
    }
    
    
    var lastTextField: UITextField? = nil
    
    func textFieldDidBeginEditing(_ textField: UITextField) {
        lastTextField = textField
    }
    

    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        textField.resignFirstResponder()
        lastTextField = textField
        return true
    }
    
    func startMainVC() {
        
        let sb = UIStoryboard(name: "Main", bundle: nil)
        if let vc = sb.instantiateInitialViewController() {
            self.present(vc, animated: true)
        }
        
    }
    
    
    func keyboardDismiss() {
        self.view.endEditing(true)
    }
    
   
    func setupViews() {
        
        let yourAttributes : [NSAttributedString.Key: Any] = [
            NSAttributedString.Key.font : UIFont.systemFont(ofSize: 14),
            NSAttributedString.Key.foregroundColor : Colors.primaryColor,
            NSAttributedString.Key.underlineStyle : NSUnderlineStyle.single.rawValue]

        let attributeString = NSMutableAttributedString(string: "Forgot password?".localized,
                                                        attributes: yourAttributes)
        forgotpassword.setAttributedTitle(attributeString, for: .normal)
        
        login_field.delegate = self
        password_field.delegate = self
        
        
        login_field.textColor = UIColor.black
        login_field.lineColor = UIColor.gray
        login_field.selectedTitleColor = UIColor.black
        login_field.selectedLineColor = Colors.primaryColor
        login_field.selectedLineHeight = 1.5
        login_field.lineHeight = 0.5
        login_field.titleColor = Colors.primaryColor
        login_field.font = UIFont(name: AppConfig.Design.Fonts.regular, size: login_field.font!.pointSize)
        login_field.titleFont = UIFont(name: AppConfig.Design.Fonts.regular, size: password_field.font!.pointSize)!
        
        
        password_field.textColor = UIColor.black
        password_field.lineColor = UIColor.gray
        password_field.selectedTitleColor = UIColor.black
        password_field.selectedLineColor = Colors.primaryColor
        password_field.selectedLineHeight = 1.5
        password_field.lineHeight = 0.5
        password_field.titleColor = Colors.primaryColor
        password_field.font = UIFont(name: AppConfig.Design.Fonts.regular, size: login_field.font!.pointSize)
        password_field.titleFont = UIFont(name: AppConfig.Design.Fonts.regular, size: password_field.font!.pointSize)!

        
        login_field.placeholder = "Enter login".localized
        password_field.placeholder = "Enter password".localized
      
        signin.setTitle("Sign In".localized.uppercased(), for: .normal)
        signup.setTitle("Don't have account?".localized.uppercased(), for: .normal)
        
        
        forgotpassword.initBoldFont()
        signup.setCustomButtonControlStyle()
        signin.setCustomButtonControlStyle(active: true)
        signin.setGradientBackground()
      
      
        login_field.delegate = self
        password_field.delegate = self
     
        self.scrollview.backgroundColor = .clear
    
        self.navigationBar.isTranslucent = true
        self.navigationBar.setBackgroundImage(UIImage(), for: UIBarMetrics.default)
        self.navigationBar.shadowImage = UIImage()
        self.navigationBar.tintColor = .white
        
        setupNavBarButtons()
        
     
    }

    
    func setupNavBarButtons() {
        
        //arrow back icon
        var arrowImage: UIImage? = nil
        if Utils.isRTL(){
            arrowImage = UIImage.init(icon: .ionicons(.iosArrowForward), size: CGSize(width: 30, height: 30), textColor: Colors.darkColor)
        }else{
            arrowImage = UIImage.init(icon: .ionicons(.iosArrowBack), size: CGSize(width: 30, height: 30), textColor: Colors.darkColor)
        }
        
        let customBarButtonItem = UIBarButtonItem(image: arrowImage!, style: .plain, target: self, action: #selector(onBackHandler))
        customBarButtonItem.setIcon(icon: .ionicons(.iosArrowBack), iconSize: 25, color: Colors.white)
    
        navigationBarItem.leftBarButtonItems = []
        navigationBarItem.rightBarButtonItems = []
        navigationBarItem.leftBarButtonItems?.append(customBarButtonItem)
        
    }
   

    var userLoader: UserLoader = UserLoader()
    func dologin () {
        
        
        MyProgress.show(parent: self.view)
        
    
        self.userLoader.delegate = self
        
        
        guard let loginValue = login_field.text else {
            showErros(messages:["login":"Login field is empty"])
            return
        }
    
        guard let passwordValue = password_field.text else {
            showErros(messages:["password":"Password field is empty"])
            return
        }
        
        
        //Get current Location
        var lat = 0.0
        var lng = 0.0
        var guest_id = 0
        
        if let guest = Guest.getInstance() {
            lat = guest.lat
            lng = guest.lng
            guest_id = guest.id
        }
        
        let parameter =  [
            "login"     : loginValue,
            "password"  : passwordValue,
            "lat"       : String(lat),
            "lng"       : String(lng),
            "guest_id"  : String(guest_id)
        ]
        
        Utils.printDebug("Parameter=>\(parameter)")
        
        self.userLoader.load(url: Constances.Api.API_USER_LOGIN,parameters: parameter)
        
        
    }
    
    
    func success(parser: UserParser,response: String) {
        
        if parser.success == 1 {
            
        
            MyProgress.showProgressWithSuccess(withStatus: "Success!")
            
            let users = parser.parse()
            
            if users.count == 1 {
                
                Utils.printDebug("\(users)")
                
                Session.createSession(user: users[0])
                
                SwiftEventBus.post("on_main_refresh", sender: users[0])
                
                
                if let del = self.delegate{
                    del.loginSuccess(controller: self, user: users[0])
                }else{
                     if MainViewController.mInstance != nil {
                        self.dismiss(animated: true)
                    }else{
                        self.startMainVC()
                    }
                }
                
            }else{
                showErros(messages: ["err": "User not found!"])
            }
            
          
            
            
        }else {
            
        
            
            if let errors = parser.errors {
                
                MyProgress.dismiss()
                self.showAlertError(title: "Error",content: errors ,msgBnt: "OK")
            }
            
        }
        
    }
    
    func error(error: Error?,response: String) {
    
        MyProgress.dismiss()
        
        let errors: [String: String] = [
            "err": "Technical error"
        ]
        
       
        self.showAlertError(title: "Error",content: errors,msgBnt: "OK")
        
        
    }
    
    
    func showErros(messages: [String: String]) {
        
         self.showAlertError(title: "Error",content: messages,msgBnt: "OK")
        
    }
    
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        self.view.endEditing(true)
    }
  


}




